"""Asset minification utilities for the Copy to LLM plugin."""

from typing import Any

from mkdocs import utils


def minify_js(content: str) -> str:
    """
    Minify JavaScript content.

    Args:
        content: JavaScript content to minify

    Returns:
        Minified JavaScript content
    """
    try:
        from jsmin import jsmin  # type: ignore[import-untyped]

        return str(jsmin(content))
    except ImportError:
        utils.log.info("jsmin not installed. Install with: pip install jsmin")
        return content
    except Exception as e:
        utils.log.error(f"Error minifying JavaScript: {e}")
        return content


def minify_css(content: str) -> str:
    """
    Minify CSS content.

    Args:
        content: CSS content to minify

    Returns:
        Minified CSS content
    """
    try:
        from csscompressor import compress  # type: ignore[import-untyped]

        return str(compress(content))
    except ImportError:
        utils.log.info(
            "csscompressor not installed. Install with: pip install csscompressor"
        )
        return content
    except Exception as e:
        utils.log.error(f"Error minifying CSS: {e}")
        return content


def should_minify(config: dict[str, Any]) -> bool:
    """
    Check if minification should be performed based on configuration.

    Args:
        config: MkDocs configuration dictionary

    Returns:
        True if minification should be performed
    """
    # Check if we're in development mode
    if config.get("dev_addr"):
        return False

    # Check plugin-specific setting
    return bool(config.get("plugins", {}).get("copy-to-llm", {}).get("minify", True))



"""Test cases for custom exceptions."""

from mkdocs_copy_to_llm.exceptions import (
    AssetError,
    AssetNotFoundError,
    AssetProcessingError,
    BuildError,
    ColorValidationError,
    ConfigurationError,
    CopyToLLMError,
    ValidationError,
)


class TestExceptionHierarchy:
    """Test exception hierarchy and inheritance."""

    def test_base_exception(self) -> None:
        """Test that CopyToLLMError is the base exception."""
        exc = CopyToLLMError("test error")
        assert isinstance(exc, Exception)
        assert str(exc) == "test error"

    def test_configuration_error(self) -> None:
        """Test ConfigurationError."""
        exc = ConfigurationError("config error")
        assert isinstance(exc, CopyToLLMError)
        assert str(exc) == "config error"

    def test_asset_errors(self) -> None:
        """Test asset-related errors."""
        exc1 = AssetError("asset error")
        assert isinstance(exc1, CopyToLLMError)

        exc2 = AssetNotFoundError("file not found")
        assert isinstance(exc2, AssetError)
        assert isinstance(exc2, CopyToLLMError)

        exc3 = AssetProcessingError("processing failed")
        assert isinstance(exc3, AssetError)
        assert isinstance(exc3, CopyToLLMError)

    def test_validation_errors(self) -> None:
        """Test validation errors."""
        exc1 = ValidationError("validation failed")
        assert isinstance(exc1, CopyToLLMError)

        exc2 = ColorValidationError("#invalid", "field_name")
        assert isinstance(exc2, ValidationError)
        assert isinstance(exc2, CopyToLLMError)
        assert "field_name" in str(exc2)
        assert "#invalid" in str(exc2)
        assert exc2.color == "#invalid"
        assert exc2.field == "field_name"

    def test_build_error(self) -> None:
        """Test BuildError."""
        exc = BuildError("build failed")
        assert isinstance(exc, CopyToLLMError)
        assert str(exc) == "build failed"


class TestColorValidationError:
    """Test ColorValidationError specific functionality."""

    def test_error_message_format(self) -> None:
        """Test that error message is properly formatted."""
        exc = ColorValidationError("not-a-color", "button_bg_color")
        error_msg = str(exc)

        assert "Invalid color format" in error_msg
        assert "not-a-color" in error_msg
        assert "button_bg_color" in error_msg
        assert "hex (#RGB or #RRGGBB)" in error_msg
        assert "rgb(r,g,b)" in error_msg
        assert "CSS color name" in error_msg
        assert "CSS variable (var(--name))" in error_msg

    def test_attributes(self) -> None:
        """Test that attributes are properly set."""
        exc = ColorValidationError("#GGG", "toast_bg_color")
        assert exc.color == "#GGG"
        assert exc.field == "toast_bg_color"



"""Test cases for the Copy to LLM plugin."""

import tempfile
from pathlib import Path
from unittest.mock import patch

import pytest
from mkdocs.config import Config

from mkdocs_copy_to_llm.exceptions import (
    AssetNotFoundError,
    AssetProcessingError,
    BuildError,
    ColorValidationError,
)
from mkdocs_copy_to_llm.plugin import CopyToLLMPlugin, CopyToLLMPluginConfig


class TestCopyToLLMPlugin:
    """Test cases for CopyToLLMPlugin."""

    def test_plugin_initialization(self) -> None:
        """Test that the plugin initializes correctly."""
        plugin = CopyToLLMPlugin()
        assert plugin.js_path == ""
        assert plugin.css_path == ""

    def test_config_defaults(self) -> None:
        """Test that the config has correct defaults."""
        config = CopyToLLMPluginConfig()
        assert config.button_bg_color == ""
        assert config.button_hover_color == ""
        assert config.toast_bg_color == ""
        assert config.toast_text_color == ""
        assert config.repo_url == ""
        assert config.minify is True
        assert config.analytics is False

    def test_on_config(self) -> None:
        """Test the on_config hook."""
        plugin = CopyToLLMPlugin()
        config = Config(schema=())

        # Call on_config
        result = plugin.on_config(config)

        # Check that extra_javascript and extra_css were added
        assert "extra_javascript" in result
        assert "extra_css" in result
        assert "assets/copy-to-llm/copy-to-llm.js" in result["extra_javascript"]
        assert "assets/copy-to-llm/copy-to-llm.css" in result["extra_css"]

    def test_custom_css_generation(self) -> None:
        """Test custom CSS generation with color configuration."""
        plugin = CopyToLLMPlugin()
        plugin.config = {
            "button_bg_color": "#ffffff",
            "button_hover_color": "#0969da",
            "toast_bg_color": "#0969da",
            "toast_text_color": "#ffffff",
        }

        css = plugin._generate_custom_css()
        assert "--copy-to-llm-button-bg: #ffffff;" in css
        assert "--copy-to-llm-button-hover: #0969da;" in css
        assert "--copy-to-llm-toast-bg: #0969da;" in css
        assert "--copy-to-llm-toast-text: #ffffff;" in css

    def test_custom_css_generation_empty(self) -> None:
        """Test custom CSS generation with no color configuration."""
        plugin = CopyToLLMPlugin()
        plugin.config = {}

        css = plugin._generate_custom_css()
        assert css == ""

    def test_on_page_content_with_repo_url(self) -> None:
        """Test the on_page_content hook with repo URL configuration."""
        plugin = CopyToLLMPlugin()
        plugin.config = {"repo_url": "https://raw.githubusercontent.com/test/repo/main"}

        html = "<head></head><body>Test</body>"
        config = Config(schema=())
        config["site_name"] = "Test Site"

        result = plugin.on_page_content(html, None, config, None)

        assert 'meta name="mkdocs-copy-to-llm-repo-url"' in result
        assert 'content="https://raw.githubusercontent.com/test/repo/main"' in result
        assert 'meta name="mkdocs-site-name"' in result
        assert 'content="Test Site"' in result

    def test_on_page_content_without_repo_url(self) -> None:
        """Test the on_page_content hook without repo URL configuration."""
        plugin = CopyToLLMPlugin()
        plugin.config = {}

        html = "<head></head><body>Test</body>"
        config = Config(schema=())
        config["site_name"] = "Test Site"

        result = plugin.on_page_content(html, None, config, None)

        assert 'meta name="mkdocs-copy-to-llm-repo-url"' not in result
        assert 'meta name="mkdocs-site-name"' in result
        assert 'content="Test Site"' in result

    def test_on_pre_build(self) -> None:
        """Test the on_pre_build hook."""
        plugin = CopyToLLMPlugin()
        plugin.config = {}

        with tempfile.TemporaryDirectory() as tmpdir:
            config = Config(schema=())
            config["docs_dir"] = tmpdir

            # Call on_pre_build
            plugin.on_pre_build(config)

            # Check that the assets directory was created
            assets_dir = Path(tmpdir) / "assets" / "copy-to-llm"
            assert assets_dir.exists()

    def test_on_post_build(self) -> None:
        """Test the on_post_build hook."""
        plugin = CopyToLLMPlugin()

        with tempfile.TemporaryDirectory() as tmpdir:
            config = Config(schema=())
            config["docs_dir"] = tmpdir

            # Create a test assets directory
            assets_dir = Path(tmpdir) / "assets" / "copy-to-llm"
            assets_dir.mkdir(parents=True, exist_ok=True)

            # Create a test file
            test_file = assets_dir / "test.txt"
            test_file.write_text("test")

            assert assets_dir.exists()

            # Call on_post_build
            plugin.on_post_build(config)

            # Check that the assets directory was removed
            assert not assets_dir.exists()

    def test_color_validation_error(self) -> None:
        """Test that invalid colors raise validation errors."""
        plugin = CopyToLLMPlugin()
        plugin.config = {"button_bg_color": "#INVALID"}

        with pytest.raises(ColorValidationError) as exc_info:
            plugin._validate_config()

        assert "button_bg_color" in str(exc_info.value)
        assert "#INVALID" in str(exc_info.value)

    def test_invalid_repo_url(self) -> None:
        """Test that invalid repository URLs raise errors."""
        plugin = CopyToLLMPlugin()
        plugin.config = {"repo_url": "not-a-url"}

        with pytest.raises(BuildError) as exc_info:
            plugin._validate_config()

        assert "Invalid repository URL" in str(exc_info.value)

    def test_on_config_with_validation_error(self) -> None:
        """Test on_config with validation error."""
        plugin = CopyToLLMPlugin()
        plugin.config = {"button_hover_color": "invalid-color"}
        config = Config(schema=())

        with pytest.raises(ColorValidationError):
            plugin.on_config(config)

    def test_on_pre_build_missing_assets(self) -> None:
        """Test on_pre_build when assets are missing."""
        plugin = CopyToLLMPlugin()
        plugin.config = {}

        with tempfile.TemporaryDirectory() as tmpdir:
            config = Config(schema=())
            config["docs_dir"] = tmpdir

            # Mock the plugin directory to a non-existent path
            with patch("os.path.dirname") as mock_dirname:
                mock_dirname.return_value = "/non/existent/path"

                with pytest.raises(AssetNotFoundError) as exc_info:
                    plugin.on_pre_build(config)

                assert "JavaScript file not found" in str(exc_info.value)

    def test_css_sanitization(self) -> None:
        """Test that CSS values are sanitized."""
        plugin = CopyToLLMPlugin()
        plugin.config = {
            "button_bg_color": "#FFFFFF; malicious: code",
            "toast_bg_color": "rgb(255, 0, 0)",
        }

        css = plugin._generate_custom_css()

        # Check that malicious code is removed
        assert "malicious" not in css
        assert "#FFFFFF" in css
        assert "rgb(255, 0, 0)" in css

    def test_analytics_meta_tag_enabled(self) -> None:
        """Test that analytics meta tag is injected when enabled."""
        plugin = CopyToLLMPlugin()
        plugin.config = {"analytics": True}

        html = "<head></head><body>Test</body>"
        config = Config(schema=())

        result = plugin.on_page_content(html, None, config, None)

        assert 'meta name="mkdocs-copy-to-llm-analytics"' in result
        assert 'content="true"' in result

    def test_analytics_meta_tag_disabled(self) -> None:
        """Test that analytics meta tag shows false when disabled."""
        plugin = CopyToLLMPlugin()
        plugin.config = {"analytics": False}

        html = "<head></head><body>Test</body>"
        config = Config(schema=())

        result = plugin.on_page_content(html, None, config, None)

        assert 'meta name="mkdocs-copy-to-llm-analytics"' in result
        assert 'content="false"' in result

    def test_color_validation_all_fields(self) -> None:
        """Test validation of all color fields."""
        plugin = CopyToLLMPlugin()

        # Test valid colors for all fields
        plugin.config = {
            "button_bg_color": "#123456",
            "button_hover_color": "rgb(255, 0, 0)",
            "toast_bg_color": "var(--primary)",
            "toast_text_color": "blue",
        }

        # Should not raise any errors
        plugin._validate_config()

    def test_color_validation_individual_fields(self) -> None:
        """Test that each color field is validated when present."""
        plugin = CopyToLLMPlugin()

        # Test button_hover_color validation
        plugin.config = {"button_hover_color": "#INVALID"}
        with pytest.raises(ColorValidationError) as exc_info:
            plugin._validate_config()
        assert "button_hover_color" in str(exc_info.value)

        # Test toast_bg_color validation
        plugin.config = {"toast_bg_color": "not-a-color-123"}
        with pytest.raises(ColorValidationError) as exc_info:
            plugin._validate_config()
        assert "toast_bg_color" in str(exc_info.value)

        # Test toast_text_color validation
        plugin.config = {"toast_text_color": "rgb(256, 256, 256)"}  # Invalid RGB
        with pytest.raises(ColorValidationError) as exc_info:
            plugin._validate_config()
        assert "toast_text_color" in str(exc_info.value)

    def test_on_config_error_handling(self) -> None:
        """Test on_config error handling and logging."""
        plugin = CopyToLLMPlugin()
        plugin.config = {"button_bg_color": "#INVALID"}
        config = Config(schema=())

        # Mock the logger to check error logging
        with patch("mkdocs_copy_to_llm.plugin.utils.log.error") as mock_log_error:
            with pytest.raises(ColorValidationError):
                plugin.on_config(config)

            # Check that error was logged
            mock_log_error.assert_called_once()
            assert "Copy to LLM plugin configuration error" in str(
                mock_log_error.call_args
            )

    def test_custom_css_generation_with_custom_css_file(self) -> None:
        """Test that custom CSS file path is added when custom CSS exists."""
        plugin = CopyToLLMPlugin()
        plugin.config = {"button_bg_color": "#FF0000"}
        config = Config(schema=())

        result = plugin.on_config(config)

        # Check that custom CSS file is added
        assert "assets/copy-to-llm/copy-to-llm-custom.css" in result["extra_css"]

    def test_on_pre_build_with_minification_disabled(self) -> None:
        """Test on_pre_build when minification is disabled."""
        plugin = CopyToLLMPlugin()
        plugin.config = {"minify": False}

        with tempfile.TemporaryDirectory() as tmpdir:
            config = Config(schema=())
            config["docs_dir"] = tmpdir

            # Create source files
            plugin_dir = Path(__file__).parent.parent / "mkdocs_copy_to_llm"
            js_src = plugin_dir / "assets" / "js" / "copy-to-llm.js"
            css_src = plugin_dir / "assets" / "css" / "copy-to-llm.css"

            if js_src.exists() and css_src.exists():
                with (
                    patch("mkdocs_copy_to_llm.plugin.minify_js") as mock_minify_js,
                    patch("mkdocs_copy_to_llm.plugin.minify_css") as mock_minify_css,
                ):
                    plugin.on_pre_build(config)

                    # Minification should not be called when disabled
                    mock_minify_js.assert_not_called()
                    mock_minify_css.assert_not_called()

    def test_on_pre_build_css_file_not_found(self) -> None:
        """Test on_pre_build when CSS file is not found."""
        plugin = CopyToLLMPlugin()
        plugin.config = {}

        with tempfile.TemporaryDirectory() as tmpdir:
            config = Config(schema=())
            config["docs_dir"] = tmpdir

            # Mock js file exists but css doesn't
            with patch("os.path.exists") as mock_exists:
                mock_exists.side_effect = lambda path: "copy-to-llm.js" in path

                with pytest.raises(AssetNotFoundError) as exc_info:
                    plugin.on_pre_build(config)

                assert "CSS file not found" in str(exc_info.value)

    def test_on_pre_build_js_processing_error(self) -> None:
        """Test on_pre_build when JS processing fails."""
        plugin = CopyToLLMPlugin()
        plugin.config = {}

        with tempfile.TemporaryDirectory() as tmpdir:
            config = Config(schema=())
            config["docs_dir"] = tmpdir

            # First create the assets directory
            assets_dir = Path(tmpdir) / "assets" / "copy-to-llm"
            assets_dir.mkdir(parents=True, exist_ok=True)

            # Create a mock scenario where file exists but reading fails
            with (
                patch("os.path.exists", return_value=True),
                patch("builtins.open", side_effect=OSError("Permission denied")),
            ):
                with pytest.raises(AssetProcessingError) as exc_info:
                    plugin.on_pre_build(config)

                assert "Failed to process JavaScript" in str(exc_info.value)

    def test_on_pre_build_css_processing_error(self) -> None:
        """Test on_pre_build when CSS processing fails."""
        plugin = CopyToLLMPlugin()
        plugin.config = {}

        with tempfile.TemporaryDirectory() as tmpdir:
            config = Config(schema=())
            config["docs_dir"] = tmpdir
            assets_dir = Path(tmpdir) / "assets" / "copy-to-llm"
            assets_dir.mkdir(parents=True, exist_ok=True)

            # Create real JS file but mock CSS processing
            plugin_dir = Path(__file__).parent.parent / "mkdocs_copy_to_llm"
            js_src = plugin_dir / "assets" / "js" / "copy-to-llm.js"
            css_src = plugin_dir / "assets" / "css" / "copy-to-llm.css"

            if js_src.exists() and css_src.exists():
                # Mock file operations
                with patch("os.path.exists") as mock_exists:
                    mock_exists.return_value = True  # All files exist

                    original_open = open

                    def open_side_effect(path, *args, **kwargs):
                        path_str = str(path)
                        path_obj = Path(path)
                        # Allow JS operations to succeed
                        if (
                            "copy-to-llm.js" in path_str
                            and "assets/copy-to-llm" in path_str
                        ):
                            # This is writing the JS file, let it succeed
                            return original_open(path, *args, **kwargs)
                        elif path_obj == js_src:
                            # This is reading the JS source, let it succeed
                            return original_open(path, *args, **kwargs)
                        # Fail CSS operations
                        elif "copy-to-llm.css" in path_str:
                            if path_obj == css_src:
                                # Allow reading CSS source
                                return original_open(path, *args, **kwargs)
                            else:
                                # Fail writing CSS
                                raise OSError("CSS write failed")
                        return original_open(path, *args, **kwargs)

                    with patch("builtins.open", side_effect=open_side_effect):
                        with pytest.raises(AssetProcessingError) as exc_info:
                            plugin.on_pre_build(config)

                        assert "Failed to process CSS" in str(exc_info.value)

    def test_on_pre_build_custom_css_creation_error(self) -> None:
        """Test on_pre_build when custom CSS file creation fails."""
        plugin = CopyToLLMPlugin()
        plugin.config = {"button_bg_color": "#FF0000"}  # This triggers custom CSS

        with tempfile.TemporaryDirectory() as tmpdir:
            config = Config(schema=())
            config["docs_dir"] = tmpdir

            # Create source files
            plugin_dir = Path(__file__).parent.parent / "mkdocs_copy_to_llm"
            js_src = plugin_dir / "assets" / "js" / "copy-to-llm.js"
            css_src = plugin_dir / "assets" / "css" / "copy-to-llm.css"

            if js_src.exists() and css_src.exists():
                # Make regular file operations succeed but custom CSS creation fail
                original_open = open
                call_count = 0

                def open_side_effect(path, *args, **kwargs):
                    nonlocal call_count
                    call_count += 1
                    if "copy-to-llm-custom.css" in str(path):
                        raise OSError("Cannot create custom CSS")
                    return original_open(path, *args, **kwargs)

                with patch("builtins.open", side_effect=open_side_effect):
                    with pytest.raises(AssetProcessingError) as exc_info:
                        plugin.on_pre_build(config)

                    assert "Failed to create custom CSS" in str(exc_info.value)

    def test_on_pre_build_unexpected_error(self) -> None:
        """Test on_pre_build with unexpected error."""
        plugin = CopyToLLMPlugin()
        plugin.config = {}

        with tempfile.TemporaryDirectory() as tmpdir:
            config = Config(schema=())
            config["docs_dir"] = tmpdir

            # Raise an unexpected error
            with patch("os.makedirs", side_effect=RuntimeError("Unexpected error")):
                with pytest.raises(BuildError) as exc_info:
                    plugin.on_pre_build(config)

                assert "Error during pre-build" in str(exc_info.value)
                assert "Unexpected error" in str(exc_info.value)

    def test_custom_css_generation_with_specific_fields(self) -> None:
        """Test CSS generation with specific color fields only."""
        plugin = CopyToLLMPlugin()

        # Test with only hover color
        plugin.config = {"button_hover_color": "#00FF00"}
        css = plugin._generate_custom_css()
        assert "--copy-to-llm-button-hover: #00FF00;" in css
        assert "--copy-to-llm-button-bg:" not in css

        # Test with only toast colors
        plugin.config = {
            "toast_bg_color": "rgb(100, 100, 100)",
            "toast_text_color": "white",
        }
        css = plugin._generate_custom_css()
        assert "--copy-to-llm-toast-bg: rgb(100, 100, 100);" in css
        assert "--copy-to-llm-toast-text: white;" in css
        assert "--copy-to-llm-button-bg:" not in css
        assert "--copy-to-llm-button-hover:" not in css

    def test_on_config_with_existing_extra_assets(self) -> None:
        """Test on_config when extra_javascript and extra_css already exist."""
        plugin = CopyToLLMPlugin()
        config = Config(schema=())

        # Pre-populate extra_javascript and extra_css
        config["extra_javascript"] = ["existing.js"]
        config["extra_css"] = ["existing.css"]

        # Call on_config
        result = plugin.on_config(config)

        # Check that our assets were appended
        assert "existing.js" in result["extra_javascript"]
        assert "existing.css" in result["extra_css"]
        assert "assets/copy-to-llm/copy-to-llm.js" in result["extra_javascript"]
        assert "assets/copy-to-llm/copy-to-llm.css" in result["extra_css"]

        # Verify order - existing assets should come first
        assert result["extra_javascript"].index("existing.js") < result[
            "extra_javascript"
        ].index("assets/copy-to-llm/copy-to-llm.js")
        assert result["extra_css"].index("existing.css") < result["extra_css"].index(
            "assets/copy-to-llm/copy-to-llm.css"
        )

    def test_on_page_content_without_head_tag(self) -> None:
        """Test on_page_content when HTML has no head tag."""
        plugin = CopyToLLMPlugin()
        plugin.config = {"repo_url": "https://example.com", "analytics": True}

        html = "<body>Test content</body>"
        config = Config(schema=())
        config["site_name"] = "Test Site"

        result = plugin.on_page_content(html, None, config, None)

        # Should return original HTML unchanged
        assert result == html
        assert "<head>" not in result
        assert "mkdocs-copy-to-llm-repo-url" not in result

    def test_on_post_build_with_missing_assets_dir(self) -> None:
        """Test on_post_build when assets directory doesn't exist."""
        plugin = CopyToLLMPlugin()

        with tempfile.TemporaryDirectory() as tmpdir:
            config = Config(schema=())
            config["docs_dir"] = tmpdir

            # Assets dir doesn't exist
            assets_dir = Path(tmpdir) / "assets" / "copy-to-llm"
            assert not assets_dir.exists()

            # Call on_post_build - should not raise any errors
            plugin.on_post_build(config)

            # Still doesn't exist
            assert not assets_dir.exists()

    def test_on_pre_build_with_custom_css_success(self) -> None:
        """Test on_pre_build successfully creates custom CSS file."""
        plugin = CopyToLLMPlugin()
        plugin.config = {
            "button_bg_color": "#FF0000",
            "button_hover_color": "#CC0000",
            "toast_bg_color": "#00FF00",
            "toast_text_color": "#FFFFFF",
        }

        with tempfile.TemporaryDirectory() as tmpdir:
            config = Config(schema=())
            config["docs_dir"] = tmpdir

            # Create source files first
            plugin_dir = Path(__file__).parent.parent / "mkdocs_copy_to_llm"
            js_src = plugin_dir / "assets" / "js" / "copy-to-llm.js"
            css_src = plugin_dir / "assets" / "css" / "copy-to-llm.css"

            # Only run test if source files exist
            if js_src.exists() and css_src.exists():
                # Call on_pre_build
                plugin.on_pre_build(config)

                # Check that custom CSS file was created
                assets_dir = Path(tmpdir) / "assets" / "copy-to-llm"
                custom_css_path = assets_dir / "copy-to-llm-custom.css"

                assert custom_css_path.exists()

                # Verify content
                content = custom_css_path.read_text()
                assert "--copy-to-llm-button-bg: #FF0000;" in content
                assert "--copy-to-llm-button-hover: #CC0000;" in content
                assert "--copy-to-llm-toast-bg: #00FF00;" in content
                assert "--copy-to-llm-toast-text: #FFFFFF;" in content



"""Test cases for validation functions."""

import pytest

from mkdocs_copy_to_llm.exceptions import ColorValidationError
from mkdocs_copy_to_llm.validation import (
    sanitize_css_value,
    validate_color,
    validate_minify_option,
    validate_url,
)


class TestColorValidation:
    """Test color validation function."""

    def test_valid_hex_colors(self) -> None:
        """Test valid hex color formats."""
        assert validate_color("#FFFFFF", "test") is True
        assert validate_color("#ffffff", "test") is True
        assert validate_color("#123456", "test") is True
        assert validate_color("#ABC", "test") is True
        assert validate_color("#abc", "test") is True
        assert validate_color("#F0F", "test") is True

    def test_valid_rgb_colors(self) -> None:
        """Test valid RGB color formats."""
        assert validate_color("rgb(255, 255, 255)", "test") is True
        assert validate_color("rgb(0,0,0)", "test") is True
        assert validate_color("rgb( 128 , 128 , 128 )", "test") is True
        assert validate_color("rgba(255, 0, 0, 0.5)", "test") is True

    def test_valid_css_variables(self) -> None:
        """Test valid CSS variable formats."""
        assert validate_color("var(--primary-color)", "test") is True
        assert validate_color("var(--md-primary-fg-color)", "test") is True
        assert validate_color("var(--theme-color-main)", "test") is True

    def test_valid_color_names(self) -> None:
        """Test valid CSS color names."""
        assert validate_color("red", "test") is True
        assert validate_color("blue", "test") is True
        assert validate_color("darkslategray", "test") is True
        assert validate_color("transparent", "test") is True

    def test_empty_color(self) -> None:
        """Test that empty string is valid."""
        assert validate_color("", "test") is True

    def test_invalid_colors(self) -> None:
        """Test invalid color formats."""
        with pytest.raises(ColorValidationError) as exc_info:
            validate_color("#GGGGGG", "test_field")
        assert "test_field" in str(exc_info.value)
        assert "#GGGGGG" in str(exc_info.value)

        with pytest.raises(ColorValidationError):
            validate_color("#12345", "test")  # Wrong length

        with pytest.raises(ColorValidationError):
            validate_color(
                "rgb(256, 0, 0)", "test"
            )  # Will pass basic regex but is invalid

        with pytest.raises(ColorValidationError):
            validate_color("var(invalid)", "test")  # Missing --

        with pytest.raises(ColorValidationError):
            validate_color("not-a-color()", "test")


class TestURLValidation:
    """Test URL validation function."""

    def test_valid_urls(self) -> None:
        """Test valid URL formats."""
        assert validate_url("https://github.com/user/repo") is True
        assert validate_url("http://example.com") is True
        assert validate_url("https://raw.githubusercontent.com/user/repo/main") is True
        assert validate_url("https://example.com:8080/path") is True
        assert validate_url("https://sub.domain.com/path/to/file") is True

    def test_empty_url(self) -> None:
        """Test that empty string is valid."""
        assert validate_url("") is True

    def test_invalid_urls(self) -> None:
        """Test invalid URL formats."""
        assert validate_url("not-a-url") is False
        assert validate_url("ftp://example.com") is False  # Only http(s)
        assert validate_url("//example.com") is False  # Missing protocol
        assert validate_url("https://") is False  # Missing domain


class TestCSSValueSanitization:
    """Test CSS value sanitization."""

    def test_valid_css_values(self) -> None:
        """Test that valid CSS values are preserved."""
        assert sanitize_css_value("#FFFFFF") == "#FFFFFF"
        assert sanitize_css_value("rgb(255, 255, 255)") == "rgb(255, 255, 255)"
        assert sanitize_css_value("var(--color)") == "var(--color)"
        assert sanitize_css_value("blue") == "blue"

    def test_dangerous_values_removed(self) -> None:
        """Test that dangerous values are sanitized."""
        assert "javascript:" not in sanitize_css_value("javascript:alert(1)")
        assert "expression" not in sanitize_css_value("expression(alert(1))")
        assert "@import" not in sanitize_css_value("@import url(evil.css)")
        assert "script" not in sanitize_css_value("</script>")
        assert "style" not in sanitize_css_value("<style>")

    def test_special_characters_removed(self) -> None:
        """Test that special characters are removed."""
        # Semicolons indicate CSS injection attempts, so everything after is removed
        assert sanitize_css_value("color; hack: value") == "color"
        assert sanitize_css_value("color/*comment*/") == "colorcomment"
        assert sanitize_css_value("url('evil')") == "url(evil)"

    def test_empty_value(self) -> None:
        """Test empty value handling."""
        assert sanitize_css_value("") == ""
        assert sanitize_css_value(None) is None


class TestMinifyValidation:
    """Test minify option validation."""

    def test_valid_boolean_values(self) -> None:
        """Test valid boolean values."""
        assert validate_minify_option(True) is True
        assert validate_minify_option(False) is True

    def test_invalid_values(self) -> None:
        """Test invalid values."""
        assert validate_minify_option("true") is False
        assert validate_minify_option(1) is False
        assert validate_minify_option(None) is False



"""Custom exceptions for the MkDocs Copy to LLM plugin."""


class CopyToLLMError(Exception):
    """Base exception for all Copy to LLM plugin errors."""

    pass


class ConfigurationError(CopyToLLMError):
    """Raised when plugin configuration is invalid."""

    pass


class AssetError(CopyToLLMError):
    """Raised when there are issues with plugin assets."""

    pass


class AssetNotFoundError(AssetError):
    """Raised when required plugin assets are not found."""

    pass


class AssetProcessingError(AssetError):
    """Raised when asset processing (e.g., minification) fails."""

    pass


class ValidationError(CopyToLLMError):
    """Raised when input validation fails."""

    pass


class ColorValidationError(ValidationError):
    """Raised when color format validation fails."""

    def __init__(self, color: str, field: str) -> None:
        super().__init__(
            f"Invalid color format '{color}' for {field}. "
            f"Expected hex (#RGB or #RRGGBB), rgb(r,g,b), CSS color name, "
            f"or CSS variable (var(--name))."
        )
        self.color = color
        self.field = field


class BuildError(CopyToLLMError):
    """Raised when there are issues during the build process."""

    pass



"""Input validation utilities for the MkDocs Copy to LLM plugin."""

import re
from typing import Any

from .exceptions import ColorValidationError


def validate_color(color: str, field_name: str) -> bool:
    """
    Validate color format.

    Args:
        color: Color value to validate
        field_name: Name of the configuration field for error messages

    Returns:
        True if color is valid

    Raises:
        ColorValidationError: If color format is invalid
    """
    if not color:  # Empty string is valid (uses default)
        return True

    # Define valid color patterns
    patterns = [
        r"^#[0-9A-Fa-f]{6}$",  # Hex 6-digit: #RRGGBB
        r"^#[0-9A-Fa-f]{3}$",  # Hex 3-digit: #RGB
        # RGB: rgb(r, g, b) with 0-255 values
        r"^rgb\(\s*(25[0-5]|2[0-4]\d|1?\d{1,2})\s*,\s*"
        r"(25[0-5]|2[0-4]\d|1?\d{1,2})\s*,\s*"
        r"(25[0-5]|2[0-4]\d|1?\d{1,2})\s*\)$",
        # RGBA with alpha 0-1
        r"^rgba\(\s*(25[0-5]|2[0-4]\d|1?\d{1,2})\s*,\s*"
        r"(25[0-5]|2[0-4]\d|1?\d{1,2})\s*,\s*"
        r"(25[0-5]|2[0-4]\d|1?\d{1,2})\s*,\s*"
        r"(0|1|0?\.\d+)\s*\)$",
        r"^var\(--[\w-]+\)$",  # CSS variable: var(--name)
        r"^[a-zA-Z]+$",  # CSS color name (basic check)
    ]

    for pattern in patterns:
        if re.match(pattern, color.strip()):
            return True

    raise ColorValidationError(color, field_name)


def validate_url(url: str) -> bool:
    """
    Validate repository URL format.

    Args:
        url: URL to validate

    Returns:
        True if URL is valid
    """
    if not url:  # Empty string is valid (no repo URL)
        return True

    # Basic URL validation - must start with http(s)://
    url_pattern = r"^https?://[\w\-.]+(:\d+)?(/.*)?$"
    return bool(re.match(url_pattern, url.strip()))


def sanitize_css_value(value: str) -> str:
    """
    Sanitize CSS value to prevent injection.

    Args:
        value: CSS value to sanitize

    Returns:
        Sanitized CSS value
    """
    if not value:
        return value

    # First, split by semicolon and take only the first part
    # This prevents CSS injection attempts
    value = value.split(";")[0].strip()

    # Remove any dangerous patterns with their content
    sanitized = value
    dangerous_patterns = [
        r"javascript:[^;]*",
        r"expression\s*\([^)]*\)",
        r"@import[^;]*",
        r"</?script[^>]*>?",
        r"</?style[^>]*>?",
    ]

    for pattern in dangerous_patterns:
        sanitized = re.sub(pattern, "", sanitized, flags=re.IGNORECASE)

    # Then remove potentially dangerous characters
    # Allow alphanumeric, spaces, parentheses, commas, hyphens, #, and periods
    sanitized = re.sub(r"[^a-zA-Z0-9\s(),#.\-]", "", sanitized)

    return sanitized.strip()


def validate_minify_option(value: Any) -> bool:
    """
    Validate minify configuration option.

    Args:
        value: Value to validate

    Returns:
        True if value is a valid boolean
    """
    return isinstance(value, bool)



"""Test cases for the minification utilities."""

from unittest.mock import MagicMock, patch

from mkdocs_copy_to_llm.minify import minify_css, minify_js, should_minify


class TestJavaScriptMinification:
    """Test cases for JavaScript minification."""

    def test_minify_js_with_jsmin_available(self) -> None:
        """Test JS minification when jsmin is available."""
        # Mock the jsmin import inside the function
        mock_jsmin = MagicMock(return_value="minified js")
        with patch.dict("sys.modules", {"jsmin": MagicMock(jsmin=mock_jsmin)}):
            result = minify_js("var x = 1;\nvar y = 2;")

            assert result == "minified js"
            mock_jsmin.assert_called_once_with("var x = 1;\nvar y = 2;")

    def test_minify_js_without_jsmin(self) -> None:
        """Test JS minification when jsmin is not available."""
        # Ensure jsmin is not in sys.modules
        with patch.dict("sys.modules", {"jsmin": None}):
            result = minify_js("var x = 1;\nvar y = 2;")

            # Should return original content
            assert result == "var x = 1;\nvar y = 2;"

    def test_minify_js_with_error(self) -> None:
        """Test JS minification when jsmin raises an error."""
        # Mock jsmin to raise an exception
        mock_jsmin = MagicMock(side_effect=Exception("Minification failed"))
        with patch.dict("sys.modules", {"jsmin": MagicMock(jsmin=mock_jsmin)}):
            result = minify_js("var x = 1;")

            # Should return original content on error
            assert result == "var x = 1;"


class TestCSSMinification:
    """Test cases for CSS minification."""

    def test_minify_css_with_csscompressor_available(self) -> None:
        """Test CSS minification when csscompressor is available."""
        # Mock the csscompressor import inside the function
        mock_compress = MagicMock(return_value="minified css")
        with patch.dict(
            "sys.modules", {"csscompressor": MagicMock(compress=mock_compress)}
        ):
            result = minify_css("body { color: red; }")

            assert result == "minified css"
            mock_compress.assert_called_once_with("body { color: red; }")

    def test_minify_css_without_csscompressor(self) -> None:
        """Test CSS minification when csscompressor is not available."""
        # Ensure csscompressor is not in sys.modules
        with patch.dict("sys.modules", {"csscompressor": None}):
            result = minify_css("body { color: red; }")

            # Should return original content
            assert result == "body { color: red; }"

    def test_minify_css_with_error(self) -> None:
        """Test CSS minification when csscompressor raises an error."""
        # Mock compress to raise an exception
        mock_compress = MagicMock(side_effect=Exception("Compression failed"))
        with patch.dict(
            "sys.modules", {"csscompressor": MagicMock(compress=mock_compress)}
        ):
            result = minify_css("body { color: red; }")

            # Should return original content on error
            assert result == "body { color: red; }"


class TestShouldMinify:
    """Test cases for should_minify function."""

    def test_should_not_minify_in_dev_mode(self) -> None:
        """Test that minification is disabled in development mode."""
        config = {"dev_addr": "127.0.0.1:8000"}

        assert should_minify(config) is False

    def test_should_minify_in_production_default(self) -> None:
        """Test that minification is enabled by default in production."""
        config = {}

        assert should_minify(config) is True

    def test_should_respect_plugin_minify_setting_true(self) -> None:
        """Test that plugin minify setting is respected when True."""
        config = {"plugins": {"copy-to-llm": {"minify": True}}}

        assert should_minify(config) is True

    def test_should_respect_plugin_minify_setting_false(self) -> None:
        """Test that plugin minify setting is respected when False."""
        config = {"plugins": {"copy-to-llm": {"minify": False}}}

        assert should_minify(config) is False

    def test_should_not_minify_in_dev_mode_even_with_plugin_setting(self) -> None:
        """Test that dev mode takes precedence over plugin setting."""
        config = {
            "dev_addr": "127.0.0.1:8000",
            "plugins": {"copy-to-llm": {"minify": True}},
        }

        assert should_minify(config) is False

    def test_should_handle_missing_plugins_config(self) -> None:
        """Test handling when plugins config is missing."""
        config = {"plugins": {}}

        assert should_minify(config) is True

    def test_should_handle_missing_copy_to_llm_config(self) -> None:
        """Test handling when copy-to-llm config is missing."""
        config = {"plugins": {"other-plugin": {}}}

        assert should_minify(config) is True



import os
import shutil
from typing import Any, Optional

from mkdocs import utils
from mkdocs.config import Config, config_options
from mkdocs.config.defaults import MkDocsConfig
from mkdocs.plugins import BasePlugin

from .exceptions import (
    AssetNotFoundError,
    AssetProcessingError,
    BuildError,
    CopyToLLMError,
)
from .minify import minify_css, minify_js, should_minify
from .validation import sanitize_css_value, validate_color, validate_url


class CopyToLLMPluginConfig(Config):
    button_bg_color = config_options.Type(str, default="")
    button_hover_color = config_options.Type(str, default="")
    toast_bg_color = config_options.Type(str, default="")
    toast_text_color = config_options.Type(str, default="")
    repo_url = config_options.Type(str, default="")
    minify = config_options.Type(bool, default=True)
    analytics = config_options.Type(bool, default=False)


class CopyToLLMPlugin(BasePlugin[CopyToLLMPluginConfig]):
    """
    MkDocs plugin to add 'Copy to LLM' buttons to documentation
    """

    def __init__(self) -> None:
        self.js_path: str = ""
        self.css_path: str = ""

    def _validate_config(self) -> None:
        """Validate plugin configuration."""
        # Validate color configurations
        if self.config.get("button_bg_color"):
            validate_color(self.config["button_bg_color"], "button_bg_color")

        if self.config.get("button_hover_color"):
            validate_color(self.config["button_hover_color"], "button_hover_color")

        if self.config.get("toast_bg_color"):
            validate_color(self.config["toast_bg_color"], "toast_bg_color")

        if self.config.get("toast_text_color"):
            validate_color(self.config["toast_text_color"], "toast_text_color")

        # Validate repository URL
        if self.config.get("repo_url") and not validate_url(self.config["repo_url"]):
            raise BuildError(f"Invalid repository URL: {self.config['repo_url']}")

    def on_config(self, config: MkDocsConfig) -> Optional[MkDocsConfig]:
        """
        Called after the user configuration is loaded
        """
        try:
            # Validate configuration
            self._validate_config()

            # Add our JS to extra_javascript if it exists
            if "extra_javascript" not in config:
                config["extra_javascript"] = []

            # Add our CSS to extra_css if it exists
            if "extra_css" not in config:
                config["extra_css"] = []

            # We'll inject these paths later when we copy the files
            self.js_path = "assets/copy-to-llm/copy-to-llm.js"
            self.css_path = "assets/copy-to-llm/copy-to-llm.css"

            config["extra_javascript"].append(self.js_path)
            config["extra_css"].append(self.css_path)

            # Add custom CSS if colors are configured
            custom_css = self._generate_custom_css()
            if custom_css:
                # Add custom CSS after our main CSS
                config["extra_css"].append("assets/copy-to-llm/copy-to-llm-custom.css")

            return config

        except Exception as e:
            utils.log.error(f"Copy to LLM plugin configuration error: {e}")
            raise

    def _generate_custom_css(self) -> str:
        """Generate custom CSS based on configuration"""
        css_parts = []

        if self.config.get("button_bg_color"):
            color = sanitize_css_value(self.config["button_bg_color"])
            css_parts.append(f"--copy-to-llm-button-bg: {color};")

        if self.config.get("button_hover_color"):
            color = sanitize_css_value(self.config["button_hover_color"])
            css_parts.append(f"--copy-to-llm-button-hover: {color};")

        if self.config.get("toast_bg_color"):
            color = sanitize_css_value(self.config["toast_bg_color"])
            css_parts.append(f"--copy-to-llm-toast-bg: {color};")

        if self.config.get("toast_text_color"):
            color = sanitize_css_value(self.config["toast_text_color"])
            css_parts.append(f"--copy-to-llm-toast-text: {color};")

        if css_parts:
            return f":root {{\n  {' '.join(css_parts)}\n}}"
        return ""

    def on_pre_build(self, config: MkDocsConfig) -> None:
        """
        Called before the build process starts
        """
        try:
            # Copy our assets to the docs directory so they're included in the build
            docs_dir: str = config["docs_dir"]
            plugin_dir: str = os.path.dirname(os.path.abspath(__file__))

            # Create an assets directory in docs
            assets_dir: str = os.path.join(docs_dir, "assets", "copy-to-llm")
            os.makedirs(assets_dir, exist_ok=True)

            # Process JS file
            js_src: str = os.path.join(plugin_dir, "assets", "js", "copy-to-llm.js")
            if not os.path.exists(js_src):
                raise AssetNotFoundError(f"JavaScript file not found: {js_src}")

            js_dest: str = os.path.join(assets_dir, "copy-to-llm.js")

            try:
                # Read and optionally minify JS
                with open(js_src, encoding="utf-8") as f:
                    js_content = f.read()

                if self.config.get("minify", True) and should_minify(dict(config)):
                    js_content = minify_js(js_content)
                    utils.log.info("Minified Copy to LLM JavaScript")

                with open(js_dest, "w", encoding="utf-8") as f:
                    f.write(js_content)

                utils.log.info(f"Processed Copy to LLM JS to {js_dest}")
            except Exception as e:
                raise AssetProcessingError(f"Failed to process JavaScript: {e}") from e

            # Process CSS file
            css_src: str = os.path.join(plugin_dir, "assets", "css", "copy-to-llm.css")
            if not os.path.exists(css_src):
                raise AssetNotFoundError(f"CSS file not found: {css_src}")

            css_dest: str = os.path.join(assets_dir, "copy-to-llm.css")

            try:
                # Read and optionally minify CSS
                with open(css_src, encoding="utf-8") as f:
                    css_content = f.read()

                if self.config.get("minify", True) and should_minify(dict(config)):
                    css_content = minify_css(css_content)
                    utils.log.info("Minified Copy to LLM CSS")

                with open(css_dest, "w", encoding="utf-8") as f:
                    f.write(css_content)

                utils.log.info(f"Processed Copy to LLM CSS to {css_dest}")
            except Exception as e:
                raise AssetProcessingError(f"Failed to process CSS: {e}") from e

            # Create a custom CSS file if needed
            custom_css = self._generate_custom_css()
            if custom_css:
                custom_css_path = os.path.join(assets_dir, "copy-to-llm-custom.css")
                try:
                    with open(custom_css_path, "w", encoding="utf-8") as f:
                        f.write(custom_css)
                    utils.log.info(f"Created custom CSS file at {custom_css_path}")
                except Exception as e:
                    raise AssetProcessingError(
                        f"Failed to create custom CSS: {e}"
                    ) from e

        except CopyToLLMError:
            # Re-raise our custom exceptions
            raise
        except Exception as e:
            # Wrap unexpected exceptions
            raise BuildError(f"Error during pre-build: {e}") from e

    def on_page_content(
        self, html: str, page: Any, config: MkDocsConfig, files: Any
    ) -> str:
        """
        Called after the page content is rendered
        """
        meta_tags = []

        if self.config.get("repo_url"):
            # Inject repository URL as a meta tag
            repo_url = self.config["repo_url"]
            meta_tags.append(
                f'<meta name="mkdocs-copy-to-llm-repo-url" content="{repo_url}">'
            )

        # Always inject the site name
        site_name = config.get("site_name", "")
        meta_tags.append(f'<meta name="mkdocs-site-name" content="{site_name}">')

        # Inject analytics configuration
        analytics_enabled = "true" if self.config.get("analytics", False) else "false"
        meta_tags.append(
            f'<meta name="mkdocs-copy-to-llm-analytics" content="{analytics_enabled}">'
        )

        # Insert all meta tags after <head> tag
        meta_tags_str = "\n".join(meta_tags)
        html = html.replace("<head>", f"<head>\n{meta_tags_str}", 1)

        return html

    def on_post_build(self, config: MkDocsConfig) -> None:
        """
        Called after the build is complete - clean up temporary files
        """
        # Clean up the temporary assets we copied to docs_dir
        docs_dir: str = config["docs_dir"]
        assets_dir: str = os.path.join(docs_dir, "assets", "copy-to-llm")

        if os.path.exists(assets_dir):
            shutil.rmtree(assets_dir)
            utils.log.info("Cleaned up temporary Copy to LLM assets")
